@@ -24,7 +24,6 @@ extern "C" {
 #endif
 
 #include "tvtimeconf.h"
-#include "console.h"
 #include "commands.h"
 
 typedef struct input_s input_t;
@@ -41,9 +40,10 @@ enum input_event
     I_REMOTE
 };
 
-input_t *input_new( config_t *cfg, commands_t *com, console_t *con, int verbose );
+input_t *input_new( config_t *cfg, commands_t *com, int verbose );
 void input_delete( input_t *in );
 void input_callback( input_t *in, int command, int arg );
+void input_xawtv_command( input_t *in, int argc, char **argv );
 
 #ifdef __cplusplus
 };
