@@ -376,18 +376,6 @@ void osd_string_composite_packed422_scanline( osd_string_t *osds,
     }
 }
 
-/* Graphic functions */
-struct osd_graphic_s
-{
-    uint8_t *image4444;
-    int image_width;
-    int image_height;
-    int frames_left;
-    int alpha;
-    int hold;
-    int *active;
-};
-
 static int load_png_to_packed4444( uint8_t *buffer, int *active,
                                    int width, int height, int stride,
                                    double pixel_aspect, pnginput_t *pngin )
@@ -442,140 +430,6 @@ static int load_png_to_packed4444( uint8_t *buffer, int *active,
     return 1;
 }
 
-osd_graphic_t *osd_graphic_new( const char *filename, double pixel_aspect, int alpha )
-{
-    osd_graphic_t *osdg = malloc( sizeof( osd_graphic_t ) );
-    char *fullfilename;
-    pnginput_t *pngin;
-
-    if( !osdg ) {
-        return 0;
-    }
-
-    osdg->hold = 0;
-    fullfilename = get_tvtime_file( filename );
-    if( !fullfilename ) {
-        fprintf( stderr, "osd_graphic: Can't find '%s'.  Checked: %s\n",
-                 filename, get_tvtime_paths() );
-        free( osdg );
-        return 0;
-    }
-    pngin = pnginput_new( fullfilename );
-    free( fullfilename );
-
-    if( !pngin ) {
-        free( osdg );
-        return 0;
-    }
-
-    osdg->frames_left = 0;
-    osdg->alpha = alpha;
-    osdg->image_width = (int) (( ((double) pnginput_get_width( pngin )) * pixel_aspect ) + 1.5);
-    osdg->image_height = pnginput_get_height( pngin );
-    osdg->image4444 = malloc( osdg->image_width * osdg->image_height * 4 );
-    if( !osdg->image4444 ) {
-        pnginput_delete( pngin );
-        free( osdg );
-        return 0;
-    }
-
-    osdg->active = malloc( osdg->image_height * sizeof( int ) );
-    if( !osdg->active ) {
-        pnginput_delete( pngin );
-        free( osdg->image4444 );
-        free( osdg );
-        return 0;
-    }
-
-    if( !load_png_to_packed4444( osdg->image4444, osdg->active, osdg->image_width,
-                                 osdg->image_height, osdg->image_width*4,
-                                 pixel_aspect, pngin ) ) {
-        fprintf( stderr, "osd_graphic: Can't render image '%s'.\n", filename );
-        pnginput_delete( pngin );
-        free( osdg->active );
-        free( osdg->image4444 );
-        free( osdg );
-        return 0;
-    }
-
-    pnginput_delete( pngin );
-    return osdg;
-}
-
-void osd_graphic_delete( osd_graphic_t *osdg )
-{
-    free( osdg->active );
-    free( osdg->image4444 );
-    free( osdg );
-}
-
-int osd_graphic_get_width( osd_graphic_t *osdg )
-{
-    return osdg->image_width;
-}
-
-int osd_graphic_get_height( osd_graphic_t *osdg )
-{
-    return osdg->image_height;
-}
-
-void osd_graphic_set_timeout( osd_graphic_t *osdg, int timeout )
-{
-    osdg->frames_left = timeout;
-}
-
-void osd_graphic_set_hold( osd_graphic_t *osdg, int hold )
-{
-    osdg->hold = hold;
-}
-
-int osd_graphic_visible( osd_graphic_t *osdg )
-{
-    return (osdg->frames_left > 0);
-}
-
-void osd_graphic_advance_frame( osd_graphic_t *osdg )
-{
-    if( osdg->frames_left > 0) {
-        if( !osdg->hold ) osdg->frames_left--;
-    }
-}
-
-int osd_graphic_active_on_scanline( osd_graphic_t *osdg, int scanline )
-{
-    if( scanline < osdg->image_height ) {
-        return osdg->active[ scanline ];
-    }
-    return 0;
-}
-
-void osd_graphic_composite_packed422_scanline( osd_graphic_t *osdg,
-                                               uint8_t *output,
-                                               uint8_t *background,
-                                               int width, int xpos,
-                                               int scanline )
-{
-    if( osdg->frames_left ) {
-        if( scanline < osdg->image_height && xpos < osdg->image_width ) {
-            int alpha;
-
-            if( (xpos+width) > osdg->image_width ) {
-                width = osdg->image_width - xpos;
-            }
-
-            if( osdg->frames_left < OSD_FADEOUT_TIME ) {
-                alpha = (int) ( ( ( ( (double) osdg->frames_left ) / ((double) OSD_FADEOUT_TIME) ) * osdg->alpha ) + 0.5 );
-            } else {
-                alpha = osdg->alpha;
-            }
-
-            composite_packed4444_alpha_to_packed422_scanline( output, background,
-                osdg->image4444 + (osdg->image_width*scanline*4) + (xpos*4),
-                width, alpha );
-        }
-    }
-}
-
 /* Graphic functions */
 struct osd_animation_s
 {
