@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2002, 2003 Billy Biggs <vektor@dumbterm.net>.
+ * Copyright (c) 2002, 2003, 2005 Billy Biggs <vektor@dumbterm.net>.
  * Copyright (C) 2001 Matthew J. Marjanovic <maddog@mir.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,38 +17,6 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/**
- * Includes 420to422, 422to444 scaling filters from the MPEG2 reference
- * implementation.  The v12 source code indicates that they were written
- * by Cheung Auyeung <auyeung@mot.com>.  The file they were in was:
- *
- * store.c, picture output routines
- * Copyright (C) 1996, MPEG Software Simulation Group. All Rights Reserved.
- *
- * Disclaimer of Warranty
- *
- * These software programs are available to the user without any license fee or
- * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
- * any and all warranties, whether express, implied, or statuary, including any
- * implied warranties or merchantability or of fitness for a particular
- * purpose.  In no event shall the copyright-holder be liable for any
- * incidental, punitive, or consequential damages of any kind whatsoever
- * arising from the use of these programs.
- *
- * This disclaimer of warranty extends to the user of these programs and user's
- * customers, employees, agents, transferees, successors, and assigns.
- *
- * The MPEG Software Simulation Group does not represent or warrant that the
- * programs furnished hereunder are free of infringement of any third-party
- * patents.
- *
- * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
- * are subject to royalty fees to patent holders.  Many of these patents are
- * general enough such that they are unavoidable regardless of implementation
- * design.
- *
- */
-
 /**
  * Code for the UYVY to YUYV routine comes from rivatv:
  *
@@ -80,19 +48,17 @@
 
 #include "speedy.h"
 #include "speedtools.h"
+#include "copyfunctions.h"
 #include "attributes.h"
 #include "mmx.h"
 #include "mm_accel.h"
 
 /* Function pointer definitions. */
-void (*interpolate_packed422_scanline)( uint8_t *output, uint8_t *top,
-                                        uint8_t *bot, int width );
 void (*blit_colour_packed422_scanline)( uint8_t *output,
                                         int width, int y, int cb, int cr );
 void (*blit_colour_packed4444_scanline)( uint8_t *output,
                                          int width, int alpha, int luma,
                                          int cb, int cr );
-void (*blit_packed422_scanline)( uint8_t *dest, const uint8_t *src, int width );
 void (*composite_packed4444_to_packed422_scanline)( uint8_t *output, uint8_t *input,
                                                     uint8_t *foreground, int width );
 void (*composite_packed4444_alpha_to_packed422_scanline)( uint8_t *output,
@@ -113,19 +79,12 @@ void (*premultiply_packed4444_scanline)( uint8_t *output, uint8_t *input, int wi
 void (*blend_packed422_scanline)( uint8_t *output, uint8_t *src1,
                                   uint8_t *src2, int width, int pos );
 unsigned int (*diff_factor_packed422_scanline)( uint8_t *cur, uint8_t *old, int width );
-unsigned int (*comb_factor_packed422_scanline)( uint8_t *top, uint8_t *mid,
-                                                uint8_t *bot, int width );
 void (*kill_chroma_packed422_inplace_scanline)( uint8_t *data, int width );
 void (*mirror_packed422_inplace_scanline)( uint8_t *data, int width );
-void (*speedy_memcpy)( void *output, const void *input, size_t size );
 void (*diff_packed422_block8x8)( pulldown_metrics_t *m, uint8_t *old,
                                  uint8_t *new, int os, int ns );
-void (*a8_subpix_blit_scanline)( uint8_t *output, uint8_t *input,
-                                 int lasta, int startpos, int width );
 void (*quarter_blit_vertical_packed422_scanline)( uint8_t *output, uint8_t *one,
                                                   uint8_t *three, int width );
-void (*subpix_blit_vertical_packed422_scanline)( uint8_t *output, uint8_t *top,
-                                                 uint8_t *bot, int subpixpos, int width );
 void (*packed444_to_nonpremultiplied_packed4444_scanline)( uint8_t *output, 
                                                            uint8_t *input,
                                                            int width, int alpha );
@@ -152,10 +111,6 @@ void (*rgba32_to_packed4444_rec601_scanline)( uint8_t *output,
                                               uint8_t *input,
                                               int width );
 void (*invert_colour_packed422_inplace_scanline)( uint8_t *data, int width );
-void (*vfilter_chroma_121_packed422_scanline)( uint8_t *output, int width,
-                                               uint8_t *m, uint8_t *t, uint8_t *b );
-void (*vfilter_chroma_332_packed422_scanline)( uint8_t *output, int width,
-                                               uint8_t *m, uint8_t *t, uint8_t *b );
 void (*convert_uyvy_to_yuyv_scanline)( uint8_t *uyvy_buf, uint8_t *yuyv_buf, int width );
 void (*composite_colour4444_alpha_to_packed422_scanline)( uint8_t *output, uint8_t *input,
                                                           int af, int y, int cb, int cr,
@@ -185,90 +140,6 @@ static inline __attribute__ ((always_inline,const)) uint8_t clip255( int x )
     }
 }
 
-unsigned long CombJaggieThreshold = 73;
-
-#ifdef ARCH_X86
-static unsigned int comb_factor_packed422_scanline_mmx( uint8_t *top, uint8_t *mid,
-                                                        uint8_t *bot, int width )
-{
-    const mmx_t qwYMask = { 0x00ff00ff00ff00ffULL };
-    const mmx_t qwOnes = { 0x0001000100010001ULL };
-    mmx_t qwThreshold;
-    unsigned int temp1, temp2;
-
-    width /= 4;
-
-    qwThreshold.uw[ 0 ] = CombJaggieThreshold;
-    qwThreshold.uw[ 1 ] = CombJaggieThreshold;
-    qwThreshold.uw[ 2 ] = CombJaggieThreshold;
-    qwThreshold.uw[ 3 ] = CombJaggieThreshold;
-
-    movq_m2r( qwThreshold, mm0 );
-    movq_m2r( qwYMask, mm1 );
-    movq_m2r( qwOnes, mm2 );
-    pxor_r2r( mm7, mm7 );         /* mm7 = 0. */
-
-    while( width-- ) {
-        /* Load and keep just the luma. */
-        movq_m2r( *top, mm3 );
-        movq_m2r( *mid, mm4 );
-        movq_m2r( *bot, mm5 );
-
-        pand_r2r( mm1, mm3 );
-        pand_r2r( mm1, mm4 );
-        pand_r2r( mm1, mm5 );
-
-        /* Work out mm6 = (top - mid) * (bot - mid) - ( (top - mid)^2 >> 7 ) */
-        psrlw_i2r( 1, mm3 );
-        psrlw_i2r( 1, mm4 );
-        psrlw_i2r( 1, mm5 );
-
-        /* mm6 = (top - mid) */
-        movq_r2r( mm3, mm6 );
-        psubw_r2r( mm4, mm6 );
-
-        /* mm3 = (top - bot) */
-        psubw_r2r( mm5, mm3 );
-
-        /* mm5 = (bot - mid) */
-        psubw_r2r( mm4, mm5 );
-
-        /* mm6 = (top - mid) * (bot - mid) */
-        pmullw_r2r( mm5, mm6 );
-
-        /* mm3 = (top - bot)^2 >> 7 */
-        pmullw_r2r( mm3, mm3 );   /* mm3 = (top - bot)^2 */
-        psrlw_i2r( 7, mm3 );      /* mm3 = ((top - bot)^2 >> 7) */
-
-        /* mm6 is what we want. */
-        psubw_r2r( mm3, mm6 );
-
-        /* FF's if greater than qwTheshold */
-        pcmpgtw_r2r( mm0, mm6 );
-
-        /* Add to count if we are greater than threshold */
-        pand_r2r( mm2, mm6 );
-        paddw_r2r( mm6, mm7 );
-
-        top += 8;
-        mid += 8;
-        bot += 8;
-    }
-
-    movd_r2m( mm7, temp1 );
-    psrlq_i2r( 32, mm7 );
-    movd_r2m( mm7, temp2 );
-    temp1 += temp2;
-    temp2 = temp1;
-    temp1 >>= 16;
-    temp1 += temp2 & 0xffff;
-
-    emms();
-
-    return temp1;
-}
-#endif
-
 static unsigned long BitShift = 6;
 
 static unsigned int diff_factor_packed422_scanline_c( uint8_t *cur, uint8_t *old, int width )
@@ -595,146 +466,6 @@ static void packed422_to_packed444_rec601_scanline_c( uint8_t *dest, uint8_t *sr
     }
 }
 
-#ifdef ARCH_X86
-static void vfilter_chroma_121_packed422_scanline_mmx( uint8_t *output, int width,
-                                                       uint8_t *m, uint8_t *t, uint8_t *b )
-{
-    int i;
-    const mmx_t ymask = { 0x00ff00ff00ff00ffULL };
-    const mmx_t cmask = { 0xff00ff00ff00ff00ULL };
-
-    // Get width in bytes.
-    width *= 2;
-    i = width / 8;
-    width -= i * 8;
-
-    movq_m2r( ymask, mm7 );
-    movq_m2r( cmask, mm6 );
-
-    while( i-- ) {
-        movq_m2r( *t, mm0 );
-        movq_m2r( *b, mm1 );
-        movq_m2r( *m, mm2 );
-
-        movq_r2r ( mm2, mm3 );
-        pand_r2r ( mm7, mm3 );
-
-        pand_r2r ( mm6, mm0 );
-        pand_r2r ( mm6, mm1 );
-        pand_r2r ( mm6, mm2 );
-
-        psrlq_i2r( 8, mm0 );
-        psrlq_i2r( 8, mm1 );
-        psrlq_i2r( 7, mm2 );
-
-        paddw_r2r( mm0, mm2 );
-        paddw_r2r( mm1, mm2 );
-
-        psllw_i2r( 6, mm2 );
-        pand_r2r( mm6, mm2 ); 
-
-        por_r2r ( mm3, mm2 );
-
-        movq_r2m( mm2, *output );
-        output += 8;
-        t += 8;
-        b += 8;
-        m += 8;
-    }
-    output++; t++; b++; m++;
-    while( width-- ) {
-        *output = (*t + *b + (*m << 1)) >> 2;
-        output+=2; t+=2; b+=2; m+=2;
-    }
-    
-    emms();
-}
-#endif
-
-static void vfilter_chroma_121_packed422_scanline_c( uint8_t *output, int width,
-                                                     uint8_t *m, uint8_t *t, uint8_t *b )
-{
-    output++; t++; b++; m++;
-    while( width-- ) {
-        *output = (*t + *b + (*m << 1)) >> 2;
-        output +=2; t+=2; b+=2; m+=2;
-    }
-}
-
-#ifdef ARCH_X86
-static void vfilter_chroma_332_packed422_scanline_mmx( uint8_t *output, int width,
-                                                       uint8_t *m, uint8_t *t, uint8_t *b )
-{
-    int i;
-    const mmx_t ymask = { 0x00ff00ff00ff00ffULL };
-    const mmx_t cmask = { 0xff00ff00ff00ff00ULL };
-
-    // Get width in bytes.
-    width *= 2; 
-    i = width / 8;
-    width -= i * 8;
-
-    movq_m2r( ymask, mm7 );
-    movq_m2r( cmask, mm6 );
-
-    while( i-- ) {
-        movq_m2r( *t, mm0 );
-        movq_m2r( *b, mm1 );
-        movq_m2r( *m, mm2 );
-
-        movq_r2r ( mm2, mm3 );
-        pand_r2r ( mm7, mm3 );
-
-        pand_r2r ( mm6, mm0 );
-        pand_r2r ( mm6, mm1 );
-        pand_r2r ( mm6, mm2 );
-
-        psrlq_i2r( 8, mm0 );
-        psrlq_i2r( 7, mm1 );
-        psrlq_i2r( 8, mm2 );
-
-        movq_r2r ( mm0, mm4 );
-        psllw_i2r( 1, mm4 );
-        paddw_r2r( mm4, mm0 );
-
-        movq_r2r ( mm2, mm4 );
-        psllw_i2r( 1, mm4 );
-        paddw_r2r( mm4, mm2 );
-
-        paddw_r2r( mm0, mm2 );
-        paddw_r2r( mm1, mm2 );
-
-        psllw_i2r( 5, mm2 );
-        pand_r2r( mm6, mm2 );
-
-        por_r2r ( mm3, mm2 );
-
-        movq_r2m( mm2, *output );
-        output += 8;
-        t += 8;
-        b += 8;
-        m += 8;
-    }
-    output++; t++; b++; m++;
-    while( width-- ) {
-        *output = (3 * *t + 3 * *m + 2 * *b) >> 3;
-        output +=2; t+=2; b+=2; m+=2;
-    }
-
-    emms();
-}
-#endif
-
-static void vfilter_chroma_332_packed422_scanline_c( uint8_t *output, int width,
-                                                     uint8_t *m, uint8_t *t, uint8_t *b )
-{
-    output++; t++; b++; m++;
-    while( width-- ) {
-        *output = (3 * *t + 3 * *m + 2 * *b) >> 3;
-        output +=2; t+=2; b+=2; m+=2;
-    }
-}
-
 #ifdef ARCH_X86
 static void kill_chroma_packed422_inplace_scanline_mmx( uint8_t *data, int width )
 {
@@ -814,16 +545,6 @@ static void mirror_packed422_inplace_scanline_c( uint8_t *data, int width )
     }
 }
 
-static void interpolate_packed422_scanline_c( uint8_t *output, uint8_t *top,
-                                              uint8_t *bot, int width )
-{
-    int i;
-
-    for( i = width*2; i; --i ) {
-        *output++ = ((*top++) + (*bot++)) >> 1;
-    }
-}
-
 #ifdef ARCH_X86
 static void convert_uyvy_to_yuyv_scanline_mmx( uint8_t *uyvy_buf, uint8_t *yuyv_buf, int width )
 {
@@ -916,130 +637,13 @@ static void convert_uyvy_to_yuyv_scanline_c( uint8_t *uyvy_buf, uint8_t *yuyv_bu
     }
 }
 
-
-#ifdef ARCH_X86
-static void interpolate_packed422_scanline_mmx( uint8_t *output, uint8_t *top,
-                                                uint8_t *bot, int width )
-{
-    const mmx_t shiftmask = { 0xfefffefffefffeffULL };  /* To avoid shifting chroma to luma. */
-    int i;
-
-    for( i = width/16; i; --i ) {
-        movq_m2r( *bot, mm0 );
-        movq_m2r( *top, mm1 );
-        movq_m2r( *(bot + 8), mm2 );
-        movq_m2r( *(top + 8), mm3 );
-        movq_m2r( *(bot + 16), mm4 );
-        movq_m2r( *(top + 16), mm5 );
-        movq_m2r( *(bot + 24), mm6 );
-        movq_m2r( *(top + 24), mm7 );
-        pand_m2r( shiftmask, mm0 );
-        pand_m2r( shiftmask, mm1 );
-        pand_m2r( shiftmask, mm2 );
-        pand_m2r( shiftmask, mm3 );
-        pand_m2r( shiftmask, mm4 );
-        pand_m2r( shiftmask, mm5 );
-        pand_m2r( shiftmask, mm6 );
-        pand_m2r( shiftmask, mm7 );
-        psrlw_i2r( 1, mm0 );
-        psrlw_i2r( 1, mm1 );
-        psrlw_i2r( 1, mm2 );
-        psrlw_i2r( 1, mm3 );
-        psrlw_i2r( 1, mm4 );
-        psrlw_i2r( 1, mm5 );
-        psrlw_i2r( 1, mm6 );
-        psrlw_i2r( 1, mm7 );
-        paddb_r2r( mm1, mm0 );
-        paddb_r2r( mm3, mm2 );
-        paddb_r2r( mm5, mm4 );
-        paddb_r2r( mm7, mm6 );
-        movq_r2m( mm0, *output );
-        movq_r2m( mm2, *(output + 8) );
-        movq_r2m( mm4, *(output + 16) );
-        movq_r2m( mm6, *(output + 24) );
-        output += 32;
-        top += 32;
-        bot += 32;
-    }
-    width = (width & 0xf);
-
-    for( i = width/4; i; --i ) {
-        movq_m2r( *bot, mm0 );
-        movq_m2r( *top, mm1 );
-        pand_m2r( shiftmask, mm0 );
-        pand_m2r( shiftmask, mm1 );
-        psrlw_i2r( 1, mm0 );
-        psrlw_i2r( 1, mm1 );
-        paddb_r2r( mm1, mm0 );
-        movq_r2m( mm0, *output );
-        output += 8;
-        top += 8;
-        bot += 8;
-    }
-    width = width & 0x7;
-
-    /* Handle last few pixels. */
-    for( i = width * 2; i; --i ) {
-        *output++ = ((*top++) + (*bot++)) >> 1;
-    }
-
-    emms();
-}
-#endif
-
-#ifdef ARCH_X86
-static void interpolate_packed422_scanline_mmxext( uint8_t *output, uint8_t *top,
-                                                   uint8_t *bot, int width )
-{
-    int i;
-
-    for( i = width/16; i; --i ) {
-        movq_m2r( *bot, mm0 );
-        movq_m2r( *top, mm1 );
-        movq_m2r( *(bot + 8), mm2 );
-        movq_m2r( *(top + 8), mm3 );
-        movq_m2r( *(bot + 16), mm4 );
-        movq_m2r( *(top + 16), mm5 );
-        movq_m2r( *(bot + 24), mm6 );
-        movq_m2r( *(top + 24), mm7 );
-        pavgb_r2r( mm1, mm0 );
-        pavgb_r2r( mm3, mm2 );
-        pavgb_r2r( mm5, mm4 );
-        pavgb_r2r( mm7, mm6 );
-        movntq_r2m( mm0, *output );
-        movntq_r2m( mm2, *(output + 8) );
-        movntq_r2m( mm4, *(output + 16) );
-        movntq_r2m( mm6, *(output + 24) );
-        output += 32;
-        top += 32;
-        bot += 32;
-    }
-    width = (width & 0xf);
-
-    for( i = width/4; i; --i ) {
-        movq_m2r( *bot, mm0 );
-        movq_m2r( *top, mm1 );
-        pavgb_r2r( mm1, mm0 );
-        movntq_r2m( mm0, *output );
-        output += 8;
-        top += 8;
-        bot += 8;
-    }
-    width = width & 0x7;
-
-    /* Handle last few pixels. */
-    for( i = width * 2; i; --i ) {
-        *output++ = ((*top++) + (*bot++)) >> 1;
-    }
-
-    sfence();
-    emms();
-}
-#endif
-
 static void blit_colour_packed422_scanline_c( uint8_t *output, int width, int y, int cb, int cr )
 {
+#ifdef WORDS_BIGENDIAN
+    uint32_t colour = cr | y << 8 | cb << 16 | y << 24;
+#else
     uint32_t colour = cr << 24 | y << 16 | cb << 8 | y;
+#endif
     uint32_t *o = (uint32_t *) output;
 
     for( width /= 2; width; --width ) {
@@ -1216,147 +820,6 @@ static void blit_colour_packed4444_scanline_mmxext( uint8_t *output, int width,
 }
 #endif
 
-
-/**
- * Some memcpy code inspired by the xine code which originally came
- * from mplayer.
- */
-
-/* linux kernel __memcpy (from: /include/asm/string.h) */
-#ifdef ARCH_X86
-static inline __attribute__ ((always_inline,const)) void small_memcpy( void *to, const void *from, size_t n )
-{
-    int d0, d1, d2;
-    __asm__ __volatile__(
-        "rep ; movsl\n\t"
-        "testb $2,%b4\n\t"
-        "je 1f\n\t"
-        "movsw\n"
-        "1:\ttestb $1,%b4\n\t"
-        "je 2f\n\t"
-        "movsb\n"
-        "2:"
-        : "=&c" (d0), "=&D" (d1), "=&S" (d2)
-        :"0" (n/4), "q" (n),"1" ((long) to),"2" ((long) from)
-        : "memory");
-}
-#endif
-
-static void speedy_memcpy_c( void *dest, const void *src, size_t n )
-{
-    if( dest != src ) {
-        memcpy( dest, src, n );
-    }
-}
-
-#ifdef ARCH_X86
-static void speedy_memcpy_mmx( void *d, const void *s, size_t n )
-{
-    const uint8_t *src = s;
-    uint8_t *dest = d;
-
-    if( dest != src ) {
-        while( n > 64 ) {
-            movq_m2r( src[ 0 ], mm0 );
-            movq_m2r( src[ 8 ], mm1 );
-            movq_m2r( src[ 16 ], mm2 );
-            movq_m2r( src[ 24 ], mm3 );
-            movq_m2r( src[ 32 ], mm4 );
-            movq_m2r( src[ 40 ], mm5 );
-            movq_m2r( src[ 48 ], mm6 );
-            movq_m2r( src[ 56 ], mm7 );
-            movq_r2m( mm0, dest[ 0 ] );
-            movq_r2m( mm1, dest[ 8 ] );
-            movq_r2m( mm2, dest[ 16 ] );
-            movq_r2m( mm3, dest[ 24 ] );
-            movq_r2m( mm4, dest[ 32 ] );
-            movq_r2m( mm5, dest[ 40 ] );
-            movq_r2m( mm6, dest[ 48 ] );
-            movq_r2m( mm7, dest[ 56 ] );
-            dest += 64;
-            src += 64;
-            n -= 64;
-        }
-
-        while( n > 8 ) {
-            movq_m2r( src[ 0 ], mm0 );
-            movq_r2m( mm0, dest[ 0 ] );
-            dest += 8;
-            src += 8;
-            n -= 8;
-        }
-
-        if( n ) small_memcpy( dest, src, n );
-
-        emms();
-    }
-}
-#endif
-
-#ifdef ARCH_X86
-static void speedy_memcpy_mmxext( void *d, const void *s, size_t n )
-{
-    const uint8_t *src = s;
-    uint8_t *dest = d;
-
-    if( dest != src ) {
-        while( n > 64 ) {
-            movq_m2r( src[ 0 ], mm0 );
-            movq_m2r( src[ 8 ], mm1 );
-            movq_m2r( src[ 16 ], mm2 );
-            movq_m2r( src[ 24 ], mm3 );
-            movq_m2r( src[ 32 ], mm4 );
-            movq_m2r( src[ 40 ], mm5 );
-            movq_m2r( src[ 48 ], mm6 );
-            movq_m2r( src[ 56 ], mm7 );
-            movntq_r2m( mm0, dest[ 0 ] );
-            movntq_r2m( mm1, dest[ 8 ] );
-            movntq_r2m( mm2, dest[ 16 ] );
-            movntq_r2m( mm3, dest[ 24 ] );
-            movntq_r2m( mm4, dest[ 32 ] );
-            movntq_r2m( mm5, dest[ 40 ] );
-            movntq_r2m( mm6, dest[ 48 ] );
-            movntq_r2m( mm7, dest[ 56 ] );
-            dest += 64;
-            src += 64;
-            n -= 64;
-        }
-
-        while( n > 8 ) {
-            movq_m2r( src[ 0 ], mm0 );
-            movntq_r2m( mm0, dest[ 0 ] );
-            dest += 8;
-            src += 8;
-            n -= 8;
-        }
-
-        if( n ) small_memcpy( dest, src, n );
-
-        sfence();
-        emms();
-    }
-}
-#endif
-
-static void blit_packed422_scanline_c( uint8_t *dest, const uint8_t *src, int width )
-{
-    speedy_memcpy_c( dest, src, width*2 );
-}
-
-#ifdef ARCH_X86
-static void blit_packed422_scanline_mmx( uint8_t *dest, const uint8_t *src, int width )
-{
-    speedy_memcpy_mmx( dest, src, width*2 );
-}
-#endif
-
-#ifdef ARCH_X86
-static void blit_packed422_scanline_mmxext( uint8_t *dest, const uint8_t *src, int width )
-{
-    speedy_memcpy_mmxext( dest, src, width*2 );
-}
-#endif
-
 static void composite_colour4444_alpha_to_packed422_scanline_c( uint8_t *output, uint8_t *input,
                                                                 int af, int y, int cb, int cr,
                                                                 int width, int alpha )
@@ -1804,7 +1267,11 @@ static void composite_alphamask_to_packed4444_scanline_c( uint8_t *output,
                                                           int textluma, int textcb,
                                                           int textcr )
 {
+#ifdef WORDS_BIGENDIAN
+    uint32_t opaque = textcr | (textcb << 8) | (textluma << 16) | (0xff << 24);
+#else
     uint32_t opaque = (textcr << 24) | (textcb << 16) | (textluma << 8) | 0xff;
+#endif
     int i;
 
     for( i = 0; i < width; i++ ) {
@@ -1813,14 +1280,27 @@ static void composite_alphamask_to_packed4444_scanline_c( uint8_t *output,
         if( a == 0xff ) {
             *((uint32_t *) output) = opaque;
         } else if( (input[ 0 ] == 0x00) ) {
+#ifdef WORDS_BIGENDIAN
+            *((uint32_t *) output) = multiply_alpha( a, textcr )
+                                       | (multiply_alpha( a, textcb ) << 8)
+                                       | (multiply_alpha( a, textluma ) << 16) | (a << 24);
+#else
             *((uint32_t *) output) = (multiply_alpha( a, textcr ) << 24)
                                        | (multiply_alpha( a, textcb ) << 16)
                                        | (multiply_alpha( a, textluma ) << 8) | a;
+#endif
         } else if( a ) {
+#ifdef WORDS_BIGENDIAN
+            *((uint32_t *) output) = (input[ 3 ] + multiply_alpha( a, textcr - input[ 3 ] ))
+                                       | ((input[ 2 ] + multiply_alpha( a, textcb - input[ 2 ] )) << 8)
+                                       | ((input[ 1 ] + multiply_alpha( a, textluma - input[ 1 ] )) << 16)
+                                       | ((input[ 0 ] + multiply_alpha( a, 0xff - input[ 0 ] )) << 24);
+#else
             *((uint32_t *) output) = ((input[ 3 ] + multiply_alpha( a, textcr - input[ 3 ] )) << 24)
                                        | ((input[ 2 ] + multiply_alpha( a, textcb - input[ 2 ] )) << 16)
                                        | ((input[ 1 ] + multiply_alpha( a, textluma - input[ 1 ] )) << 8)
                                        |  (input[ 0 ] + multiply_alpha( a, 0xff - input[ 0 ] ));
+#endif
         }
         mask++;
         output += 4;
@@ -1933,7 +1413,11 @@ static void composite_alphamask_alpha_to_packed4444_scanline_c( uint8_t *output,
                                                                 int textluma, int textcb,
                                                                 int textcr, int alpha )
 {
+#ifdef WORDS_BIGENDIAN
+    uint32_t opaque = textcr | (textcb << 8) | (textluma << 16) | (0xff << 24);
+#else
     uint32_t opaque = (textcr << 24) | (textcb << 16) | (textluma << 8) | 0xff;
+#endif
     int i;
 
     for( i = 0; i < width; i++ ) {
@@ -1945,14 +1429,27 @@ static void composite_alphamask_alpha_to_packed4444_scanline_c( uint8_t *output,
            if( a == 0xff ) {
                *((uint32_t *) output) = opaque;
            } else if( input[ 0 ] == 0x00 ) {
+#ifdef WORDS_BIGENDIAN
+               *((uint32_t *) output) = multiply_alpha( a, textcr )
+                                          | (multiply_alpha( a, textcb ) << 8)
+                                          | (multiply_alpha( a, textluma ) << 16) | (a << 24);
+#else
                *((uint32_t *) output) = (multiply_alpha( a, textcr ) << 24)
                                           | (multiply_alpha( a, textcb ) << 16)
                                           | (multiply_alpha( a, textluma ) << 8) | a;
+#endif
            } else if( a ) {
+#ifdef WORDS_BIGENDIAN
+               *((uint32_t *) output) = (input[ 3 ] + multiply_alpha( a, textcr - input[ 3 ] ))
+                                         | ((input[ 2 ] + multiply_alpha( a, textcb - input[ 2 ] )) << 8)
+                                         | ((input[ 1 ] + multiply_alpha( a, textluma - input[ 1 ] )) << 16)
+                                         | ((a + multiply_alpha( 0xff - a, input[ 0 ] )) << 24);
+#else
                *((uint32_t *) output) = ((input[ 3 ] + multiply_alpha( a, textcr - input[ 3 ] )) << 24)
                                          | ((input[ 2 ] + multiply_alpha( a, textcb - input[ 2 ] )) << 16)
                                          | ((input[ 1 ] + multiply_alpha( a, textluma - input[ 1 ] )) << 8)
                                          | (a + multiply_alpha( 0xff - a, input[ 0 ] ));
+#endif
            }
         }
         mask++;
@@ -1966,10 +1463,17 @@ static void premultiply_packed4444_scanline_c( uint8_t *output, uint8_t *input,
     while( width-- ) {
         unsigned int cur_a = input[ 0 ];
 
+#ifdef WORDS_BIGENDIAN
+        *((uint32_t *) output) = multiply_alpha( cur_a, input[ 3 ] )
+                               | (multiply_alpha( cur_a, input[ 2 ] ) << 8)
+                               | (multiply_alpha( cur_a, input[ 1 ] ) << 16)
+                               | (cur_a << 24);
+#else
         *((uint32_t *) output) = (multiply_alpha( cur_a, input[ 3 ] ) << 24)
                                | (multiply_alpha( cur_a, input[ 2 ] ) << 16)
                                | (multiply_alpha( cur_a, input[ 1 ] ) << 8)
                                | cur_a;
+#endif
 
         output += 4;
         input += 4;
@@ -2146,38 +1650,6 @@ static void quarter_blit_vertical_packed422_scanline_c( uint8_t *output, uint8_t
     }
 }
 
-static void subpix_blit_vertical_packed422_scanline_c( uint8_t *output, uint8_t *top,
-                                                       uint8_t *bot, int subpixpos, int width )
-{
-    if( subpixpos == 32768 ) {
-        interpolate_packed422_scanline( output, top, bot, width );
-    } else if( subpixpos == 16384 ) {
-        quarter_blit_vertical_packed422_scanline( output, top, bot, width );
-    } else if( subpixpos == 49152 ) {
-        quarter_blit_vertical_packed422_scanline( output, bot, top, width );
-    } else {
-        int x;
-
-        width *= 2;
-        for( x = 0; x < width; x++ ) {
-            output[ x ] = ( ( top[ x ] * subpixpos ) + ( bot[ x ] * ( 0xffff - subpixpos ) ) ) >> 16;
-        }
-    }
-}
-
-static void a8_subpix_blit_scanline_c( uint8_t *output, uint8_t *input,
-                                       int lasta, int startpos, int width )
-{
-    int pos = 0xffff - (startpos & 0xffff);
-    int prev = lasta;
-    int x;
-
-    for( x = 0; x < width; x++ ) {
-        output[ x ] = ( ( prev * pos ) + ( input[ x ] * ( 0xffff - pos ) ) ) >> 16;
-        prev = input[ x ];
-    }
-}
-
 /**
  * These are from lavtools in mjpegtools:
  *
@@ -2482,25 +1954,21 @@ void setup_speedy_calls( uint32_t accel, int verbose )
 {
     speedy_accel = accel;
 
-    interpolate_packed422_scanline = interpolate_packed422_scanline_c;
+    setup_copyfunctions( accel );
+
     blit_colour_packed422_scanline = blit_colour_packed422_scanline_c;
     blit_colour_packed4444_scanline = blit_colour_packed4444_scanline_c;
-    blit_packed422_scanline = blit_packed422_scanline_c;
     composite_packed4444_to_packed422_scanline = composite_packed4444_to_packed422_scanline_c;
     composite_packed4444_alpha_to_packed422_scanline = composite_packed4444_alpha_to_packed422_scanline_c;
     composite_alphamask_to_packed4444_scanline = composite_alphamask_to_packed4444_scanline_c;
     composite_alphamask_alpha_to_packed4444_scanline = composite_alphamask_alpha_to_packed4444_scanline_c;
     premultiply_packed4444_scanline = premultiply_packed4444_scanline_c;
     blend_packed422_scanline = blend_packed422_scanline_c;
-    comb_factor_packed422_scanline = 0;
     diff_factor_packed422_scanline = diff_factor_packed422_scanline_c;
     kill_chroma_packed422_inplace_scanline = kill_chroma_packed422_inplace_scanline_c;
     mirror_packed422_inplace_scanline = mirror_packed422_inplace_scanline_c;
-    speedy_memcpy = speedy_memcpy_c;
     diff_packed422_block8x8 = diff_packed422_block8x8_c;
-    a8_subpix_blit_scanline = a8_subpix_blit_scanline_c;
     quarter_blit_vertical_packed422_scanline = quarter_blit_vertical_packed422_scanline_c;
-    subpix_blit_vertical_packed422_scanline = subpix_blit_vertical_packed422_scanline_c;
     packed444_to_nonpremultiplied_packed4444_scanline = packed444_to_nonpremultiplied_packed4444_scanline_c;
     aspect_adjust_packed4444_scanline = aspect_adjust_packed4444_scanline_c;
     packed444_to_packed422_scanline = packed444_to_packed422_scanline_c;
@@ -2510,8 +1978,6 @@ void setup_speedy_calls( uint32_t accel, int verbose )
     rgb24_to_packed444_rec601_scanline = rgb24_to_packed444_rec601_scanline_c;
     rgba32_to_packed4444_rec601_scanline = rgba32_to_packed4444_rec601_scanline_c;
     invert_colour_packed422_inplace_scanline = invert_colour_packed422_inplace_scanline_c;
-    vfilter_chroma_121_packed422_scanline = vfilter_chroma_121_packed422_scanline_c;
-    vfilter_chroma_332_packed422_scanline = vfilter_chroma_332_packed422_scanline_c;
     convert_uyvy_to_yuyv_scanline = convert_uyvy_to_yuyv_scanline_c;
     composite_colour4444_alpha_to_packed422_scanline = composite_colour4444_alpha_to_packed422_scanline_c;
 
@@ -2520,10 +1986,8 @@ void setup_speedy_calls( uint32_t accel, int verbose )
         if( verbose ) {
             fprintf( stderr, "speedycode: Using MMXEXT optimized functions.\n" );
         }
-        interpolate_packed422_scanline = interpolate_packed422_scanline_mmxext;
         blit_colour_packed422_scanline = blit_colour_packed422_scanline_mmxext;
         blit_colour_packed4444_scanline = blit_colour_packed4444_scanline_mmxext;
-        blit_packed422_scanline = blit_packed422_scanline_mmxext;
         composite_packed4444_to_packed422_scanline = composite_packed4444_to_packed422_scanline_mmxext;
         composite_packed4444_alpha_to_packed422_scanline = composite_packed4444_alpha_to_packed422_scanline_mmxext;
         composite_alphamask_to_packed4444_scanline = composite_alphamask_to_packed4444_scanline_mmxext;
@@ -2531,32 +1995,22 @@ void setup_speedy_calls( uint32_t accel, int verbose )
         kill_chroma_packed422_inplace_scanline = kill_chroma_packed422_inplace_scanline_mmx;
         blend_packed422_scanline = blend_packed422_scanline_mmxext;
         diff_factor_packed422_scanline = diff_factor_packed422_scanline_mmx;
-        comb_factor_packed422_scanline = comb_factor_packed422_scanline_mmx;
         diff_packed422_block8x8 = diff_packed422_block8x8_mmx;
         quarter_blit_vertical_packed422_scanline = quarter_blit_vertical_packed422_scanline_mmxext;
         invert_colour_packed422_inplace_scanline = invert_colour_packed422_inplace_scanline_mmx;
-        vfilter_chroma_121_packed422_scanline = vfilter_chroma_121_packed422_scanline_mmx;
-        vfilter_chroma_332_packed422_scanline = vfilter_chroma_332_packed422_scanline_mmx;
         convert_uyvy_to_yuyv_scanline = convert_uyvy_to_yuyv_scanline_mmx;
         composite_colour4444_alpha_to_packed422_scanline = composite_colour4444_alpha_to_packed422_scanline_mmxext;
-        speedy_memcpy = speedy_memcpy_mmxext;
     } else if( speedy_accel & MM_ACCEL_X86_MMX ) {
         if( verbose ) {
             fprintf( stderr, "speedycode: Using MMX optimized functions.\n" );
         }
-        interpolate_packed422_scanline = interpolate_packed422_scanline_mmx;
         blit_colour_packed422_scanline = blit_colour_packed422_scanline_mmx;
         blit_colour_packed4444_scanline = blit_colour_packed4444_scanline_mmx;
-        blit_packed422_scanline = blit_packed422_scanline_mmx;
         diff_factor_packed422_scanline = diff_factor_packed422_scanline_mmx;
-        comb_factor_packed422_scanline = comb_factor_packed422_scanline_mmx;
         kill_chroma_packed422_inplace_scanline = kill_chroma_packed422_inplace_scanline_mmx;
         diff_packed422_block8x8 = diff_packed422_block8x8_mmx;
         invert_colour_packed422_inplace_scanline = invert_colour_packed422_inplace_scanline_mmx;
-        vfilter_chroma_121_packed422_scanline = vfilter_chroma_121_packed422_scanline_mmx;
-        vfilter_chroma_332_packed422_scanline = vfilter_chroma_332_packed422_scanline_mmx;
         convert_uyvy_to_yuyv_scanline = convert_uyvy_to_yuyv_scanline_mmx;
-        speedy_memcpy = speedy_memcpy_mmx;
     } else {
         if( verbose ) {
             fprintf( stderr, "speedycode: No MMX or MMXEXT support detected, using C fallbacks.\n" );
