@@ -56,6 +56,7 @@ struct config_s
     char *geometry;
     int verbose;
     int aspect;
+    int squarepixels;
     int debug;
     int fullscreen;
     int alwaysontop;
@@ -281,6 +282,10 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             }
         }
 
+        if( !xmlStrcasecmp( name, BAD_CAST "SquarePixels" ) ) {
+            ct->squarepixels = atoi( curval );
+        }
+
         if( !xmlStrcasecmp( name, BAD_CAST "Widescreen" ) ) {
             ct->aspect = atoi( curval );
         }
@@ -596,6 +601,7 @@ static void print_usage( char **argv )
               "                                 us-cable100\n"
               "                                 us-broadcast\n"
               "                                 china-broadcast\n"
+              "                                 southafrica\n"
               "                                 japan-cable\n"
               "                                 japan-broadcast\n"
               "                                 europe\n"
@@ -647,6 +653,7 @@ static void print_config_usage( char **argv )
               "                                 us-cable100\n"
               "                                 us-broadcast\n"
               "                                 china-broadcast\n"
+              "                                 southafrica\n"
               "                                 japan-cable\n"
               "                                 japan-broadcast\n"
               "                                 europe\n"
@@ -705,6 +712,7 @@ config_t *config_new( void )
     ct->geometry = strdup( "0x576" );
     ct->verbose = 0;
     ct->aspect = 0;
+    ct->squarepixels = 1;
     ct->debug = 0;
     ct->fullscreen = 0;
     ct->alwaysontop = 0;
@@ -724,7 +732,7 @@ config_t *config_new( void )
     ct->quiet_screenshots = 0;
     ct->unmute_volume = -1;
     ct->muted = 0;
-    ct->mute_on_exit = 1;
+    ct->mute_on_exit = 0;
     ct->show_taglines = 1;
 
     memset( ct->keymap, 0, sizeof( ct->keymap ) );
@@ -796,6 +804,7 @@ config_t *config_new( void )
     ct->keymap[ I_F6 ] = TVTIME_PICTURE_DOWN;
     ct->keymap[ I_F7 ] = TVTIME_PICTURE_UP;
     ct->keymap[ 'd' ] = TVTIME_SHOW_STATS;
+    ct->keymap[ 'a' ] = TVTIME_TOGGLE_ASPECT;
     ct->keymap[ 'f' ] = TVTIME_TOGGLE_FULLSCREEN;
     ct->keymap[ 'i' ] = TVTIME_TOGGLE_INPUT;
     ct->keymap[ 's' ] = TVTIME_SCREENSHOT;
@@ -1647,3 +1656,8 @@ int config_get_show_taglines( config_t *ct )
     return ct->show_taglines;
 }
 
+int config_get_square_pixels( config_t *ct )
+{
+    return ct->squarepixels;
+}
+
