@@ -68,7 +68,7 @@ void tvtime_osd_set_audio_mode( tvtime_osd_t *osd, const char *audiomode );
 void tvtime_osd_set_input( tvtime_osd_t *osd, const char *text );
 void tvtime_osd_set_channel_number( tvtime_osd_t *osd, const char *text );
 void tvtime_osd_set_channel_name( tvtime_osd_t *osd, const char *text );
-void tvtime_osd_set_hold_message( tvtime_osd_t *osd, const char *text );
+void tvtime_osd_set_hold_message( tvtime_osd_t *osd, const char *text, const char *text2 );
 void tvtime_osd_set_deinterlace_method( tvtime_osd_t *osd, const char *method );
 void tvtime_osd_set_film_mode( tvtime_osd_t *osd, int mode );
 void tvtime_osd_set_info_available( tvtime_osd_t *osd, int mode );
