@@ -19,15 +19,22 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <libxml/tree.h>
 #include "xmltv.h"
 
+#define MAX_LANGUAGES 10
+
+typedef struct language_s language_t;
 typedef struct program_s program_t;
 
 struct xmltv_s
 {
     xmlDocPtr doc;
     xmlNodePtr root;
+    int num_languages;
+    language_t *languages;
+    language_t *locale;
     char curchannel[ 256 ];
     int refresh;
     xmlChar *curchan;
@@ -37,14 +44,26 @@ struct xmltv_s
     int is_tv_grab_na;
 };
 
+/**
+ * Struct to store language info.
+ */
+struct language_s {
+    xmlChar code[3];
+    const char *name;
+    language_t *next;
+};
+
 /**
  * Struct to store program info.
  */
 struct program_s {
     xmlChar *title;
+    xmlChar *title_local;
     xmlChar *subtitle;
+    xmlChar *subtitle_local;
     xmlChar *description;
-    char times[ 256 ];
+    xmlChar *description_local;
+    time_t start_time;
     time_t end_time;
 };
 
@@ -93,6 +112,64 @@ tz_map_t date_manip_timezones[] = {
 
 const int num_timezones = sizeof( date_manip_timezones ) / sizeof( tz_map_t );
 
+/**
+ * ISO 639 language codes
+ */
+typedef struct {
+    const char *code;
+    const char *name;
+} locale_t;
+
+static locale_t locale_table[] = {
+    {"AA", "Afar"},           {"AB", "Abkhazian"},      {"AF", "Afrikaans"},
+    {"AM", "Amharic"},        {"AR", "Arabic"},         {"AS", "Assamese"},
+    {"AY", "Aymara"},         {"AZ", "Azerbaijani"},    {"BA", "Bashkir"},
+    {"BE", "Byelorussian"},   {"BG", "Bulgarian"},      {"BH", "Bihari"},
+    {"BI", "Bislama"},        {"BN", "Bengali"},        {"BO", "Tibetan"},
+    {"BR", "Breton"},         {"CA", "Catalan"},        {"CO", "Corsican"},
+    {"CS", "Czech"},          {"CY", "Welsh"},          {"DA", "Danish"},
+    {"DE", "German"},         {"DZ", "Bhutani"},        {"EL", "Greek"},
+    {"EN", "English"},        {"EO", "Esperanto"},      {"ES", "Spanish"},
+    {"ET", "Estonian"},       {"EU", "Basque"},         {"FA", "Persian"},
+    {"FI", "Finnish"},        {"FJ", "Fiji"},           {"FO", "Faeroese"},
+    {"FR", "French"},         {"FY", "Frisian"},        {"GA", "Irish"},
+    {"GD", "Gaelic"},         {"GL", "Galician"},       {"GN", "Guarani"},
+    {"GU", "Gujarati"},       {"HA", "Hausa"},          {"HI", "Hindi"},
+    {"HR", "Croatian"},       {"HU", "Hungarian"},      {"HY", "Armenian"},
+    {"IA", "Interlingua"},    {"IE", "Interlingue"},    {"IK", "Inupiak"},
+    {"IN", "Indonesian"},     {"IS", "Icelandic"},      {"IT", "Italian"},
+    {"IW", "Hebrew"},         {"JA", "Japanese"},       {"JI", "Yiddish"},
+    {"JW", "Javanese"},       {"KA", "Georgian"},       {"KK", "Kazakh"},
+    {"KL", "Greenlandic"},    {"KM", "Cambodian"},      {"KN", "Kannada"},
+    {"KO", "Korean"},         {"KS", "Kashmiri"},       {"KU", "Kurdish"},
+    {"KY", "Kirghiz"},        {"LA", "Latin"},          {"LN", "Lingala"},
+    {"LO", "Laothian"},       {"LT", "Lithuanian"},     {"LV", "Latvian"},
+    {"MG", "Malagasy"},       {"MI", "Maori"},          {"MK", "Macedonian"},
+    {"ML", "Malayalam"},      {"MN", "Mongolian"},      {"MO", "Moldavian"},
+    {"MR", "Marathi"},        {"MS", "Malay"},          {"MT", "Maltese"},
+    {"MY", "Burmese"},        {"NA", "Nauru"},          {"NE", "Nepali"},
+    {"NL", "Dutch"},          {"NO", "Norwegian"},      {"OC", "Occitan"},
+    {"OM", "Oromo"},          {"OR", "Oriya"},          {"PA", "Punjabi"},
+    {"PL", "Polish"},         {"PS", "Pashto"},         {"PT", "Portuguese"},
+    {"QU", "Quechua"},        {"RM", "Rhaeto-Romance"}, {"RN", "Kirundi"},
+    {"RO", "Romanian"},       {"RU", "Russian"},        {"RW", "Kinyarwanda"},
+    {"SA", "Sanskrit"},       {"SD", "Sindhi"},         {"SG", "Sangro"},
+    {"SH", "Serbo-Croatian"}, {"SI", "Singhalese"},     {"SK", "Slovak"},
+    {"SL", "Slovenian"},      {"SM", "Samoan"},         {"SN", "Shona"},
+    {"SO", "Somali"},         {"SQ", "Albanian"},       {"SR", "Serbian"},
+    {"SS", "Siswati"},        {"ST", "Sesotho"},        {"SU", "Sudanese"},
+    {"SV", "Swedish"},        {"SW", "Swahili"},        {"TA", "Tamil"},
+    {"TE", "Tegulu"},         {"TG", "Tajik"},          {"TH", "Thai"},
+    {"TI", "Tigrinya"},       {"TK", "Turkmen"},        {"TL", "Tagalog"},
+    {"TN", "Setswana"},       {"TO", "Tonga"},          {"TR", "Turkish"},
+    {"TS", "Tsonga"},         {"TT", "Tatar"},          {"TW", "Twi"},
+    {"UK", "Ukrainian"},      {"UR", "Urdu"},           {"UZ", "Uzbek"},
+    {"VI", "Vietnamese"},     {"VO", "Volapuk"},        {"WO", "Wolof"},
+    {"XH", "Xhosa"},          {"YO", "Yoruba"},         {"ZH", "Chinese"},
+    {"ZU", "Zulu"} };
+
+const int num_locales = sizeof( locale_table ) / sizeof( locale_t );
+
 /**
  * Timezone parsing code based loosely on the algorithm in
  * filldata.cpp of MythTV.
@@ -132,9 +209,10 @@ static time_t parse_xmltv_date( const char *date )
     char shour[ 3 ];
     char smin[ 3 ];
     char ssec[ 3 ];
-    int len = strlen( date );
+    int len;
     struct tm tm_obj;
-    time_t tz = 0;
+    time_t tz;
+    const char *tzstring;
     /*
      * For some reason, mktime() accepts broken-time arguments as localtime,
      * and there is no corresponding UTC function. *Sigh*.
@@ -152,18 +230,45 @@ static time_t parse_xmltv_date( const char *date )
     memset( ssec, 0, sizeof( ssec ) );
 
     /*
+     * according to the xmltv dtd:
+     *
+     * All dates and times in this [the xmltv] DTD follow the same format,
+     * loosely based on ISO 8601.  They can be 'YYYYMMDDhhmmss' or some
+     * initial substring, for example if you only know the year and month you
+     * can have 'YYYYMM'.  You can also append a timezone to the end; if no
+     * explicit timezone is given, UT is assumed.  Examples:
+     * '200007281733 BST', '200209', '19880523083000 +0300'.  (BST == +0100.)
+     *
+     * thus:
      * example *date = "20031022220000 +0200"
      * type:            YYYYMMDDhhmmss ZZzzz"
      * position:        0         1         2          
      *                  012345678901234567890
+     *
+     * note: since part of the time specification can be omitted, we cannot
+     *       hard-code the offset to the timezone!
      */
+
+    /* Find where the timezone starts */
+    tzstring = strchr( date, ' ' );
+    if( tzstring ) {
+        /* Calculate the length of the date */
+        len = tzstring - date;
+        /* Parse the timezone, skipping the initial space */
+        tz = parse_xmltv_timezone( tzstring + 1 );
+    } else {
+        /* No timezone present */
+        len = strlen( date );
+        /* Assume UT */
+        tz = 0;
+    }
+
     if( len >= 4 ) memcpy( syear, date, 4 );
     if( len >= 6 ) memcpy( smonth, date + 4, 2 );
     if( len >= 8 ) memcpy( sday, date + 6, 2 );
     if( len >= 10 ) memcpy( shour, date + 8, 2 );
     if( len >= 12 ) memcpy( smin, date + 10, 2 );
     if( len >= 14 ) memcpy( ssec, date + 12, 2 );
-    if( len >= 15 ) tz = parse_xmltv_timezone( date + 15 ); else tz = 0;
 
     tm_obj.tm_sec = atoi( ssec ) - tz + gmtoff;
     tm_obj.tm_min = atoi( smin );
@@ -176,15 +281,14 @@ static time_t parse_xmltv_date( const char *date )
     return mktime( &tm_obj );
 }
 
-static void reinit_program( program_t *pro, xmlNodePtr cur )
+static void reinit_program( program_t *pro, xmlNodePtr cur, xmlChar *locale )
 {
     if( cur ) {
         xmlChar *start = xmlGetProp( cur, BAD_CAST "start" );
         xmlChar *stop = xmlGetProp( cur, BAD_CAST "stop" );
+        xmlChar *lang;
         time_t start_time = 0;
         time_t end_time = 0;
-        struct tm start_tm;
-        struct tm end_tm;
 
         if( start ) {
             start_time = parse_xmltv_date( (char *) start );
@@ -196,44 +300,60 @@ static void reinit_program( program_t *pro, xmlNodePtr cur )
         } else {
             end_time = start_time + 1800;
         }
+        pro->start_time = start_time;
         pro->end_time = end_time;
 
-        localtime_r( &start_time, &start_tm );
-        localtime_r( &end_time, &end_tm );
-
-        /* FIXME: This needs i18n-approval. In French, for example, times
-           are written as 18h30, and in Swedish, we write 18.30. Not to
-           mention different leading-zero rules. */
-        /* Printing the string here was a hack to begin with.  There
-           should be functions to return the start/end time from the
-           xmltv object so that formatting can be handled by the
-           application.  -Billy */
-        sprintf( pro->times, "%2d:%02d - %2d:%02d",
-                 start_tm.tm_hour, start_tm.tm_min,
-                 end_tm.tm_hour, end_tm.tm_min );
-
         cur = cur->xmlChildrenNode;
         while( cur ) {
+            lang = xmlGetProp( cur, BAD_CAST "lang");
             if( !xmlStrcasecmp( cur->name, BAD_CAST "title" ) ) {
-                pro->title = xmlNodeGetContent( cur );
+                if( !lang || !locale ) {
+                    if ( pro->title ) xmlFree ( pro->title );
+                    pro->title = xmlNodeGetContent( cur );
+                } else if( !xmlStrncasecmp( lang, locale, 2 ) ) {
+                    if ( pro->title_local ) xmlFree ( pro->title_local );
+                    pro->title_local = xmlNodeGetContent( cur );
+                } else if( !pro->title ) {
+                    pro->title = xmlNodeGetContent( cur );
+                }
             } else if( !xmlStrcasecmp( cur->name, BAD_CAST "sub-title" ) ) {
-                pro->subtitle = xmlNodeGetContent( cur );
+                if( !lang || !locale ) {
+                    if ( pro->subtitle ) xmlFree ( pro->subtitle );
+                    pro->subtitle = xmlNodeGetContent( cur );
+                } else if( !xmlStrncasecmp( lang, locale, 2 ) ) {
+                    if ( pro->subtitle_local )
+                        xmlFree ( pro->subtitle_local );
+                    pro->subtitle_local = xmlNodeGetContent( cur );
+                } else if( !pro->subtitle ) {
+                    pro->subtitle = xmlNodeGetContent( cur );
+                }
             } else if( !xmlStrcasecmp( cur->name, BAD_CAST "desc" ) ) {
-                pro->description = xmlNodeGetContent( cur );
+                if( !lang || !locale ) {
+                    if ( pro->description ) xmlFree ( pro->description );
+                    pro->description = xmlNodeGetContent( cur );
+                } else if( !xmlStrncasecmp( lang, locale, 2 ) ) {
+                    if ( pro->description_local )
+                        xmlFree ( pro->description_local );
+                    pro->description_local = xmlNodeGetContent( cur );
+                } else if( !pro->description ) {
+                    pro->description = xmlNodeGetContent( cur );
+                }
             }
+            if( lang ) xmlFree( lang );
             cur = cur->next;
         }
     } else {
          pro->title = pro->subtitle = pro->description = 0;
+         pro->title_local = pro->subtitle_local = pro->description_local = 0;
+         pro->start_time = 0;
          pro->end_time = 0;
-         *pro->times = 0;
     }
 }
 
 static program_t *program_new( void )
 {
     program_t *pro = malloc( sizeof( program_t ) );
-    reinit_program( pro, 0 );
+    reinit_program( pro, 0, 0 );
     return pro;
 }
 
@@ -297,7 +417,64 @@ static int xmltv_is_tv_grab_na( xmltv_t *xmltv )
     return 0;
 }
 
-xmltv_t *xmltv_new( const char *filename )
+static language_t *xmltv_add_language( xmltv_t *xmltv, const xmlChar *lang )
+{
+    language_t *last = 0;
+    language_t *l = xmltv->languages;
+    int i;
+
+    if( xmltv->num_languages >= MAX_LANGUAGES ) return 0;
+
+    while( l ) {
+        i = xmlStrncasecmp( l->code, lang, 2 );
+        if( i == 0 ) {
+            return l;
+        } else if( i < 0 ) {
+            last = l;
+            l = l->next;
+        } else {
+            break;
+        }
+    }
+
+    xmltv->num_languages++;
+    l = malloc( sizeof( language_t ) );
+    if( !l ) return 0;
+    if( last ) {
+        l->next = last->next;
+        last->next = l;
+    } else {
+        l->next = xmltv->languages;
+        xmltv->languages = l;
+    }
+    for( i=0; i<2 && isalpha(lang[i]); i++ ) {
+        l->code[i] = tolower( lang[i] );
+    }
+    l->code[i] = '\0';
+
+    for( i=0; i<num_locales; i++ ) {
+        if( !xmlStrncasecmp( lang, BAD_CAST locale_table[i].code, 2 ) ) {
+            l->name = locale_table[i].name;
+            return l;
+        }
+    }
+    l->name = 0;
+    return l;
+}
+
+static int xmltv_find_languages( xmlNodePtr node, xmltv_t *xmltv )
+{
+    xmlNodePtr cur = node->xmlChildrenNode;
+    while( cur ) {
+        xmlChar *c = xmlGetProp( cur, BAD_CAST "lang" );
+        if( c ) xmltv_add_language( xmltv, c );
+        xmltv_find_languages( cur, xmltv );
+        cur = cur->next;
+    }
+    return 0;
+}
+
+xmltv_t *xmltv_new( const char *filename, const char *locale )
 {
     xmltv_t *xmltv = malloc( sizeof( xmltv_t ) );
 
@@ -338,6 +515,15 @@ xmltv_t *xmltv_new( const char *filename )
     xmltv->refresh = 1;
     xmltv->is_tv_grab_na = xmltv_is_tv_grab_na( xmltv );
 
+    xmltv->num_languages = 0;
+    xmltv->languages = 0;
+    if( locale ) {
+        xmltv->locale = xmltv_add_language( xmltv, BAD_CAST locale );
+    } else {
+        xmltv->locale = 0;
+    }
+    xmltv_find_languages( xmltv->root, xmltv );
+
     return xmltv;
 }
 
@@ -346,10 +532,21 @@ static void program_delete( program_t *pro )
     if( pro->title ) xmlFree( pro->title );
     if( pro->subtitle ) xmlFree( pro->subtitle );
     if( pro->description ) xmlFree( pro->description );
-    reinit_program( pro, 0 );
+    reinit_program( pro, 0, 0 );
     free( pro );
 }
 
+static void languages_delete( language_t *languages )
+{
+    language_t *cur, *next;
+    cur = languages;
+    while( cur ) {
+        next = cur->next;
+        free( cur );
+        cur = next;
+    }
+}
+
 void xmltv_delete( xmltv_t *xmltv )
 {
     program_delete( xmltv->pro );
@@ -357,6 +554,7 @@ void xmltv_delete( xmltv_t *xmltv )
     if( xmltv->curchan ) xmlFree( xmltv->curchan );
     if( xmltv->display_chan ) xmlFree( xmltv->display_chan );
     xmlFreeDoc( xmltv->doc );
+    languages_delete( xmltv->languages );
     free( xmltv );
 }
 
@@ -378,23 +576,23 @@ void xmltv_refresh( xmltv_t *xmltv )
     if( xmltv->pro->title ) xmlFree( xmltv->pro->title );
     if( xmltv->pro->subtitle ) xmlFree( xmltv->pro->subtitle );
     if( xmltv->pro->description ) xmlFree( xmltv->pro->description );
-    reinit_program( xmltv->pro, 0 );
+    reinit_program( xmltv->pro, 0, 0 );
 
     if( xmltv->next_pro->title ) xmlFree( xmltv->next_pro->title );
     if( xmltv->next_pro->subtitle ) xmlFree( xmltv->next_pro->subtitle );
     if( xmltv->next_pro->description ) xmlFree( xmltv->next_pro->description );
-    reinit_program( xmltv->next_pro, 0 );
+    reinit_program( xmltv->next_pro, 0, 0 );
 
     if( *xmltv->curchannel ) {
         xmlNodePtr program_node = 0;
         program_node = get_program( xmltv->root, xmltv->curchannel, curtime );
-        reinit_program( xmltv->pro, program_node );
+        reinit_program( xmltv->pro, program_node, xmltv->locale->code );
 
         if( program_node ) {
             /* If we found a program, lookup what's on next. */
             program_node = get_program( xmltv->root, xmltv->curchannel,
                                         xmltv->pro->end_time );
-            reinit_program( xmltv->next_pro, program_node );
+            reinit_program( xmltv->next_pro, program_node, xmltv->locale->code );
         } else {
             /* We found no program, schedule a check in a half hour. */
             xmltv->pro->end_time = curtime + 1800;
@@ -405,26 +603,43 @@ void xmltv_refresh( xmltv_t *xmltv )
 
 const char *xmltv_get_title( xmltv_t *xmltv )
 {
+    if ( xmltv->pro->title_local ) {
+        return (char *) xmltv->pro->title_local;
+    }
     return (char *) xmltv->pro->title;
 }
 
 const char *xmltv_get_sub_title( xmltv_t *xmltv )
 {
+    if ( xmltv->pro->subtitle_local ) {
+        return (char *) xmltv->pro->subtitle_local;
+    }
     return (char *) xmltv->pro->subtitle;
 }
 
 const char *xmltv_get_description( xmltv_t *xmltv )
 {
+    if ( xmltv->pro->description_local ) {
+        return (char *) xmltv->pro->description_local;
+    }
     return (char *) xmltv->pro->description;
 }
 
-const char *xmltv_get_times( xmltv_t *xmltv )
+time_t xmltv_get_start_time( xmltv_t *xmltv )
 {
-    return xmltv->pro->times;
+    return xmltv->pro->start_time;
+}
+
+time_t xmltv_get_end_time( xmltv_t *xmltv )
+{
+    return xmltv->pro->end_time;
 }
 
 const char *xmltv_get_next_title( xmltv_t *xmltv )
 {
+    if ( xmltv->next_pro->title_local ) {
+        return (char *) xmltv->next_pro->title_local;
+    }
     return (char *) xmltv->next_pro->title;
 }
 
@@ -520,3 +735,70 @@ const char *xmltv_lookup_channel_name( xmltv_t *xmltv, const char *id )
     return 0;
 }
 
+void xmltv_set_language( xmltv_t *xmltv, const char *locale )
+{
+    language_t *l;
+    xmltv->refresh = 1;
+    if( !locale ) {
+        xmltv->locale = 0;
+        return;
+    }
+    l = xmltv_add_language( xmltv, BAD_CAST locale );
+    if( l ) xmltv->locale = l;
+}
+
+const char *xmltv_get_language( xmltv_t *xmltv )
+{
+    if( xmltv->locale ) return (char *) xmltv->locale->code;
+    return 0;
+}
+
+int xmltv_get_num_languages( xmltv_t *xmltv )
+{
+    return xmltv->num_languages;
+}
+
+void xmltv_select_language( xmltv_t *xmltv, int n )
+{
+    int i;
+    if( n > xmltv->num_languages ) return;
+    xmltv->refresh = 1;
+    if( n == 0 ) {
+        xmltv->locale = 0;
+        return;
+    }
+    xmltv->locale = xmltv->languages;
+    for( i=1; i<n; i++ ) xmltv->locale = xmltv->locale->next;
+}
+
+int xmltv_get_langnum( xmltv_t *xmltv )
+{
+    language_t *l = xmltv->languages;
+    int i = 1;
+    if( !xmltv->locale ) return 0;
+    while( l ) {
+        if( l == xmltv->locale ) return i;
+        l = l->next;
+        i++;
+    }
+    return 0; /* should never happen */
+}
+
+const char *xmltv_get_language_code( xmltv_t *xmltv, int n )
+{
+    language_t *l = xmltv->languages;
+    int i;
+    if( n==0 || n>xmltv->num_languages ) return 0;
+    for( i=1; i<n; i++ ) l = l->next;
+    return (char *) l->code;
+}
+
+const char *xmltv_get_language_name( xmltv_t *xmltv, int n )
+{
+    language_t *l = xmltv->languages;
+    int i;
+    if( n==0 || n>xmltv->num_languages ) return 0;
+    for( i=1; i<n; i++ ) l = l->next;
+    return (char *) l->name;
+}
+
